/*
    This file is part of the Cheap Matrix library
    Copyright (C) 2000  Nicolas Brodu
    nicolas.brodu@free.fr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program, see the file COPYING; if not, write
    to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
    MA 02139, USA.
*/
#include "CheapMatrix.h"
#include "CM_lapack.h" // We could also include only CM_svd in this example

using namespace CheapMatrix;

main()
{
	// Let's test the Singular Value Decomposition
	Matrix A(4,4);
	Matrix U(3,3);
	Matrix VT(3,3);
	Vector S(3);

	A = 1, 5, 9, 0,
	    4, 7, 8, 9,
	    2, 3, 6, 9,
	    8, 6, 2, 4;
//	cin >> A;

	// LAPACK only work with 'contiguous' matrices, all in memory.
	// That is, each column elements must have consecutive addresses in memory
	// as the row index increases along a column. The offset (calculated
	// in elements, equivalent to having the columns only partly filled) from
	// the begining of a column to the next is called the leading dimension.

	// This first example shows a SVD on a submatrix, with leading dimension
	// different (4) than the row dimension (3). Not that you have to worry
	// about it, those values are found automatically.
	bool b = SVD(A(1, 3, 1, 3), U, S, VT);
	if (!b) cerr << "You just found a bug! Please report it." << endl;
	else cout << "SVD worked." << endl;
	cout << "U:" << endl << U;
	cout << "S:" << endl << S;
	cout << "VT:" << endl << VT;

	// Lapack destroys the matrix, so reset it
	A = 1, 5, 9, 0,
	    4, 7, 8, 9,
	    2, 3, 6, 9,
	    8, 6, 2, 4;

	// So we make a non-trivial change in coordinate that still leave the
	// resulting matrix contiguous, and LAPACK should be able to work on it.
	Matrix B = A(1, 4, 2, 4);
	B = B.reshape(1,12);
	Matrix C = B(1,1,1,9);
	C = C.reshape(3,3);
	b = SVD(C, U, S, VT);
	if (!b) cerr << "You just found a bug! Please report it." << endl;
	else cout << "SVD worked." << endl;
	cout << "U:" << endl << U;
	cout << "S:" << endl << S;
	cout << "VT:" << endl << VT;
}
/* The result should be:
SVD worked.
U:
-0.611358 0.690091 -0.387318
-0.672811 -0.710934 -0.204691
-0.416612 0.135452 0.898936
S:
16.6596
2.56584
0.935764
VT:
-0.248256 -0.541209 -0.803406
-0.73377 -0.436395 0.520712
0.632417 -0.718785 0.288785
SVD worked.
U:
-0.546954 0.770721 0.326851
-0.68749 -0.190732 -0.700699
-0.477702 -0.607956 0.634184
S:
16.7025
4.72334
1.64783
VT:
-0.537661 -0.795732 -0.278802
0.147059 -0.414095 0.898275
-0.830237 0.441967 0.339663
*/
