/*
    This file is part of the Cheap Matrix library
    Copyright (C) 2000  Nicolas Brodu
    nicolas.brodu@free.fr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program, see the file COPYING; if not, write
    to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
    MA 02139, USA.
*/

// This example illustrates the danger of side effects when using references

#include "CheapMatrix.h"

using namespace CheapMatrix;

main()
{
	// Create a matrix in its own memory buffer
	Matrix a(4,4);
	a = 1, 5, 9, 0,
	    4, 7, 8, 9,
		2, 3, 6, 9,
		7, 5, 3, 1;

	// Get a reference to the same data
	Matrix b = a(2,4,2,4).transpose();

	// Naively copy b back into a
	for (int i=1; i<=3; i++) for (int j=1; j<=3; j++)
		a(i+1,j+1) = b(i,j);
	cout << a << endl;

	// Let's start over
	a = 1, 5, 9, 0,
	    4, 7, 8, 9,
		2, 3, 6, 9,
		7, 5, 3, 1;
	b = a(2,4,2,4).transpose(); // Not necessary, but I won't screw you up...

	// But this time do it properly (and it looks much nicer!)
	a(2,4,2,4).copy(b);
	cout << a << endl;

	// The copy function always takes care of side effects. If you cannot
	// use it, or for some reason don't want to, your other possibility
	// is to test a.equal_ref(b) before your loop and taking appropriate
	// action, like using a temporary or detaching a or b.
}

/* This program should print
1 5 9 0
4 7 3 5
2 3 6 3
7 5 3 1

1 5 9 0
4 7 3 5
2 8 6 3
7 9 9 1

*/
