/*
    This file is part of the Cheap Matrix library
    Copyright (C) 2000  Nicolas Brodu
    nicolas.brodu@free.fr

    The code for scaled conjugate gradients is inspired from a Netlab,
    a Matlab neural network toolbox written by
    Ian T Nabney & Christopher M Bishop (1996, 1997, 1998)
    It is available at http://www.ncrg.aston.ac.uk/netlab/index.html

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef CM_SCG_H
#define CM_SCG_H

#include "CM_optimizer.h"

namespace CheapMatrix {

// Optimize using scaled conjugate gradients and stop after niter iterations
// It returns the final sum of square error
// Options: 1: the number of iterations (default = 100)
//          2: precision on the values  (default = 1e-10)
//          3: precision on the error   (default = 1e-10)
class SCG : public Optimizer
{
protected:
	MatrixType algorithm(OptimizeTarget& opta, const Matrix& data, const Matrix& target, const Vector& options);
public:
	inline SCG(const Vector& options) : Optimizer(options) {}
	inline SCG() {}
	Vector default_options();
};

}
#endif
